# This program is free software; you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation; either version 2 of the License, or
#     (at your option) any later version.
# 
#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
# 
#     You should have received a copy of the GNU General Public License
#     along with this program; if not, write to the Free Software
#     Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

require 'Qt'
require 'ewimp.rb'

class VideoTools < Qt::Object

    MP2TMP = "mp2tmp.mp2"
    WAVTMP = "wavtmp.wav"
    GSMTMP = "gsmtmp.wav"
    MPGTMP = "mpgtmp.mpg"
    HEADERTMP = "header.tmp"

    attr_reader :afr, :abr, :ach, :acdc, :vw, :vh, :vbr, :vfps, :vq, :ivw, :ivh, :ivbpp, :ivfps, :ivkbps, :ivtime

    signals 'startEncoding(const QString&)','videoInfo(const QString&)','audioInfo(const QString&)','dpgInfo(const QString&)','siError(const QString&)','sivideoProgressBar(int)','siaudioProgressBar(int)','refresh()'

    slots 'encode(const QString&)'

    def encode()
	
    end

    def vidinfo(videoname)
	vargs = ["mplayer","-vo dummy -identify",'"'+videoname+'"']
	$stderr.puts vargs.join(" ")
	IO.popen(vargs.join(" ")).readlines.each do |line|
		$stderr.puts line
		vf = /VIDEO:.+ (\d+)x(\d+).+ (\d+)bpp.+ (\d+).(\d+) fps.+ (\d+).(\d+) kbps/.match(line) # \d+bpp \d+.\d+ fps \d+.\d+ kbps/.match(line)
		if vf != nil
			$stderr.puts "VIDEOINFORMATION!!!="+vf.to_s
			@ivw = vf[1]
			@ivh = vf[2]
			@ivbpp = vf[3]
			@ivfps = (vf[4].to_s+"."+vf[5].to_s).to_f
			@ivkbps = (vf[6].to_s+"."+vf[7].to_s).to_f	
		end
		vf = nil
		vf = /ID_LENGTH=(\d+).(\d+)/.match(line)
		if vf != nil
			@ivtime = (vf[1].to_s+"."+vf[2].to_s).to_f
		end
		emit refresh()
		
    	end
	if((@ivw != nil) && (@ivh != nil))
		$stderr.puts "OK"
		return 0
	else
		$stderr.puts "ERROR INQUIRING VIDEO INFORMATION"
		return 1
	end
    end

    def setvalues(afr, abr, ach, acdc, vw,vh, vbr, vfps, vq)
    	@afr=afr
	@abr=abr
	@ach=ach
	@acdc=acdc
	@vw=vw
	@vh=vh
	@vbr=vbr
	@vfps=vfps
	if (@vfps.to_s == "Auto")
            @vfps = (737280/(@vw.to_i*@vh.to_i)).round
        end
	@vq=vq
	@audiotmp = nil
	@totalframes = nil
    end

    def encodevideo(videoname)
	emit startEncoding(videoname)
	emit videoInfo("mencoder video encoding, please wait...")

	videoArgs = videoArgs1 = ["mencoder",'"'+videoname+'"',"-v","-ofps",@vfps.to_s,"-vf scale="+@vw.to_s+":"+@vh.to_s,"-nosound","-ovc lavc -lavcopts vcodec=mpeg1video:vstrict=-2:vbitrate="+@vbr.to_s,"-o",MPGTMP,"-of rawvideo"]
	
	videoArgs2 = ["mencoder",'"'+videoname+'"',"-v","-ofps",@vfps.to_s,"-sws 9 -vf scale="+@vw.to_s+":"+@vh.to_s+":::3","-nosound","-ovc lavc -lavcopts vcodec=mpeg1video:vstrict=-2:mbd=2:trell:cbp:mv0:cmp=2:subcmp=2:precmp=2:vbitrate="+@vbr.to_s,"-o",MPGTMP,"-of rawvideo"]
	
	videoArgs3 = ["mencoder",'"'+videoname+'"',"-v","-ofps",@vfps.to_s,"-sws 9 -vf scale="+@vw.to_s+":"+@vh.to_s+":::3","-nosound","-ovc lavc -lavcopts vcodec=mpeg1video:vstrict=-2:mbd=2:trell:cbp:mv0:cmp=6:subcmp=6:precmp=6:dia=3:predia=3:last_pred=3:vbitrate="+@vbr.to_s,"-o",MPGTMP,"-of rawvideo"]

	if (vq == 0)
		videoArgs = videoArgs1
	end
	if (vq == 1)
		videoArgs = videoArgs2
	end 
	if (vq == 2)
		videoArgs = videoArgs3
	end
	
	#Subtitles
	extpos = videoname.rindex(".")
	subs = Dir.glob(videoname[0,extpos]+"{.srt,.sub}")
	if subs[0] != nil
        	videoArgs = videoArgs + ["-sub",'"'+subs[0]+'"']
	end

	
	emit siError(videoArgs.join(" "))

	pio = IO.popen(videoArgs.join(" "))
	pio.each("\r") {|line| 
		percent = /Pos.+\((\d+)%/.match(line)
		if percent != nil
			emit sivideoProgressBar(percent[1].to_i)
		end
		match = /Video stream.+ (\d+).(\d+)\s+secs/.match(line)
		if match != nil
			frames = (((match[1].to_s+"."+match[2].to_s).to_f) * @vfps.to_i).to_i
			@totalframes = frames
		end
		emit refresh()
	 }
	Process.waitpid(pio.pid)
	
	
	if $?.to_i != 0
		emit videoInfo("ERROR ENCODING VIDEO!:"+$?.to_s)
		emit siError("ERROR encoding "+videoname)
		return 1
	else
		emit videoInfo("OK")
		emit sivideoProgressBar(100)
		return 0
	end
    end

    def encodeAudioMP2(videoname)
	emit audioInfo("mencoder MP2 audio encoding, please wait...")
	audioArgs = [ "mencoder",'"'+videoname+'"',"-v -af resample="+@afr.to_s+":1:2 -of rawaudio -oac lavc -ovc copy -lavcopts acodec=mp2:abitrate="+@abr.to_s+" -o",MP2TMP ]
	pio = IO.popen(audioArgs.join(" "))
	pio.each("\r") {|line| 
		percent = /Pos.+\((\d+)%/.match(line)
		if percent != nil
			emit siaudioProgressBar(percent[1].to_i)
		end
	 }
	Process.waitpid(pio.pid)
	
	if $?.to_i != 0
		emit audioInfo("ERROR ENCODING MP2 AUDIO!:"+$?.to_s)
		emit siError("ERROR encoding MP2 audio :"+videoname)
		return 1
	else
		emit videoInfo("OK")
		emit siaudioProgressBar(100)
		@audiotmp = MP2TMP
		return 0
	end
    end

    def encodeAudioGSM(videoname)
	emit audioInfo("ffmpeg WAV audio encoding, please wait...")
	audioArgs = [ "ffmpeg", "-y",
			"-i " + '"' + videoname + '"',
			"-vn",
			"-ar " +@afr.to_s , 
			"-ac",@ach.to_s,
			WAVTMP ]
	if ! Kernel.system(audioArgs.join(" "))
		$stderr.puts "Error extracting audio"
		emit audioInfo("ffmpeg WAV audio encoding, ERROR")
		@errormsg ="ERROR WAV encoding!: "+videoname
                emit siError(@errormsg)
		return
	else
		emit siaudioProgressBar(50)
	end

	# convert audio to gsm
	emit audioInfo("Sox GSM audio encoding, please wait...")
	soxArgs = [ "sox", WAVTMP,
			"-r " + @afr.to_s,
			"-c",@ach.to_s,
			"-g", GSMTMP ]
	if ! Kernel.system(soxArgs.join(" "))
		$stderr.puts "Error converting audio to gsm"
		emit audioInfo("sox GSM audio encoding, ERROR")
		@errormsg ="ERROR Sox GSM encoding!: "+videoname
                emit siError(@errormsg)
		return 1
	else
		emit audioInfo("GSM audio encoding, OK")
		emit siaudioProgressBar(100)
		@audiotmp = GSMTMP
		return 0
	end
    end

    def createHeaderGSM()
	emit dpgInfo("Creating header file, please wait...")
	audiosize = File.size(GSMTMP)
	videosize = File.size(MPGTMP)
	
	audiostart = 36
	audioend = audiostart + audiosize
	videostart = audioend 
	videoend = videostart + videosize

	# header
	fps = @vfps.to_i
	outfile = File.new(HEADERTMP, "wb")
	
	headerValues = [ "DPG0", @totalframes.to_i, fps, 0, @afr.to_i, @ach.to_i ]
	headerPack = "A4VnnVV"

	outfile.write(headerValues.pack(headerPack) )
	# file positions
	locationValues = [ audiostart, audioend, videostart, videoend ]
	locationPack = "VVVV"
	outfile.write( locationValues.pack(locationPack) )
	outfile.close()
    end

    def createHeaderMP2()
	emit dpgInfo("Creating header file, please wait...")
	audiosize = File.size(MP2TMP)
	videosize = File.size(MPGTMP)

	audiostart = 36
	audioend = audiosize
	videostart = audioend + 36
	videoend = videosize
	# header
	fps = @vfps.to_i
	outfile = File.new(HEADERTMP, "wb")
	
	headerValues = [ "DPG0", @totalframes.to_i, fps, 0, @afr.to_i, 0 ]
	headerPack = "A4VnnVV"

	outfile.write(headerValues.pack(headerPack) )
	# file positions
	locationValues = [ audiostart, audioend, videostart, videoend ]
	locationPack = "VVVV"
	outfile.write( locationValues.pack(locationPack) )
	outfile.close()
    end

    def concatDpg(videoname)
	emit dpgInfo("Creating dpg file, please wait...")
	aux = videoname.split("/")
	outfilename = aux[aux.size-1]
	outfilename[outfilename.size-4, outfilename.size-1]=".dpg"
	concatArgs = [ "cat ", HEADERTMP+" ",@audiotmp.to_s+" ",MPGTMP," > ",'"'+outfilename+'"']
	#$stderr.puts videoArgs.join(" ")
	Kernel.system(concatArgs.join(" "))
	emit dpgInfo("OK")
    end

    def cleanTemp()
	if FileTest.exist?(MP2TMP)
		File.delete(MP2TMP)
	end
	if FileTest.exist?(WAVTMP)
		File.delete(WAVTMP)
	end
	if FileTest.exist?(GSMTMP)
		File.delete(GSMTMP)
	end
	if FileTest.exist?(MPGTMP)
		File.delete(MPGTMP)
	end
	if FileTest.exist?(HEADERTMP)
		File.delete(HEADERTMP)
	end
    end

    def initialize()
	super
	setvalues(32000, 128, 1, "MP2", 256,192, 160, 15, 1)
    end

##TESTING CODE
#     def createDpg(videoname)
# 	error = 0
# 	emit dpgInfo("Creating dpg file, please wait...")
# 	audiosize = File.size(GTMP)
# 	videosize = File.size(VTMP)
# 	audiostart = 36
# 	aux = videoname.split("/")
# 	outfilename = aux[aux.size-1]
# 	outfilename[outfilename.size-4, outfilename.size-1]=".dpg"
# 	emit siError("1");
# 	# form header
# 	#begin
# 		#outfile = File.new(HTMP, "wb")
# # 	rescue
# # 		$stderr.puts "Error opening output file"
# # 		error=1
# # 		emit siError("Error opening new DPG file: "+videoname)
#  	#end
# 	
# 	audiostart = 36
# 	audioend = audiostart + audiosize
# 	videostart = audioend + 36
# 	videoend = videostart + videosize
# 
# 	# header
# 	#frames = 3729
# 	fps = comboBox6.currentText().to_i
# 	headerValues = [ "DPG0", @@frames, fps, 0, comboBox1.currentText().to_i, ACHANNELS ]
# 	headerPack = "A4VnnVV"
# 
# 	#outfile.write headerValues.pack(headerPack) 
# 
# 	# file positions
# 	locationValues = [ audiostart, audioend, videostart, videoend ]
# 	locationPack = "VVVV"
# emit siError("3");
# 	#outfile.write( locationValues.pack(locationPack) )
# 	#outfile.close()
# 	concatArgs = [ "cat ", HTMP+" ",GTMP+" ",VTMP," >",outfilename]
# 	$stderr.puts videoArgs.join(" ")
# 	Kernel.system(videoArgs.join(" "))
# # 	begin
# # 		audiofile = File.new(GTMP, "rb")
# # 		while data = audiofile.sysread(8192)
# # 			outfile.write(data)
# # 		end
# # 	rescue 
# # 		audiofile.close
# # 	rescue SystemCallError
# # 		puts "Error building final dpg file"
# # 	end
# # 	
# # 	begin
# # 		videofile = File.new(VTMP, "rb")
# # 		while data = videofile.sysread(8192)
# # 			outfile.write(data)
# # 		end
# # 	rescue
# # 		videofile.close
# # 	rescue SystemCallError
# # 		puts "Error building final dpg file"
# # 	end
# emit siError("4");
# 
# 	if error==0
# 		emit dpgInfo("OK")
# 	else
# 		emit dpgInfo("ERROR!")
# 	end
# emit siError("5");
# 
# 	File.delete(VTMP);
# 	File.delete(VTMP2);
# 	File.delete(GTMP);
# 	File.delete(HTMP);
# emit siError("6");
# 
#     end
end